from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.generics import ListAPIView

from uzbekistan.models import Region, District, Village, check_model
from uzbekistan.serializers import RegionModelSerializer, DistrictModelSerializer
from uzbekistan.filters import RegionFilterSet, DistrictFilterSet, VillageFilterSet


class RegionListAPIView(ListAPIView):
    serializer_class = RegionModelSerializer
    filter_backends = (DjangoFilterBackend,)
    filterset_class = RegionFilterSet
    pagination_class = None
    url_path = 'regions'
    url_name = 'region-list'
    url_relation = None

    def get_queryset(self):
        check_model(Region)
        return Region.objects.all()


class DistrictListAPIView(ListAPIView):
    serializer_class = DistrictModelSerializer
    filter_backends = (DjangoFilterBackend,)
    filterset_class = DistrictFilterSet
    pagination_class = None
    url_path = 'districts'
    url_name = 'district-list'
    url_relation = 'region_id'

    def get_queryset(self):
        check_model(District)
        return District.objects.filter(region_id=self.kwargs['region_id'])


class VillageListAPIView(ListAPIView):
    serializer_class = DistrictModelSerializer
    filter_backends = (DjangoFilterBackend,)
    filterset_class = VillageFilterSet
    pagination_class = None
    url_path = 'villages'
    url_name = 'village-list'
    url_relation = 'district_id'

    def get_queryset(self):
        check_model(Village)
        return Village.objects.filter(district_id=self.kwargs['district_id'])



from django.conf import settings
from django.contrib import admin
from django.contrib.admin import ModelAdmin

from uzbekistan.models import Region, District, Village

if settings.UZBEKISTAN['models'].get('region', False):
    @admin.register(Region)
    class RegionAdmin(ModelAdmin):
        list_display = ('name_uz', 'name_oz', 'name_ru', 'name_en')
        search_fields = ('name_uz', 'name_oz', 'name_ru', 'name_en')
        sortable_by = ('name_uz', 'name_oz', 'name_ru', 'name_en')

if settings.UZBEKISTAN['models'].get('district', False):
    @admin.register(District)
    class DistrictAdmin(ModelAdmin):
        list_display = ('name_uz', 'name_oz', 'name_ru', 'name_en', 'get_region_name')
        search_fields = ('name_uz', 'name_oz', 'name_ru', 'name_en', 'get_region_name')
        sortable_by = ('name_uz', 'name_oz', 'name_ru', 'name_en', 'region')
        save_on_top = True

        def get_region_name(self, obj):
            return obj.region.name_uz

        get_region_name.short_description = 'Region'

if settings.UZBEKISTAN['models'].get('village', False):
    @admin.register(Village)
    class VillageAdmin(ModelAdmin):
        list_display = ('name_uz', 'name_oz', 'name_ru', 'get_district_name', 'get_region_name')
        search_fields = ('name_uz', 'name_oz', 'name_ru', 'get_district_name', 'get_region_name')
        sortable_by = ('name_uz', 'name_oz', 'name_ru', 'district')
        save_on_top = True

        def get_district_name(self, obj):
            return obj.district.name_uz

        get_district_name.short_description = 'District'

        def get_region_name(self, obj):
            return obj.district.region.name_uz

        get_region_name.short_description = 'Region'



from django.urls import path

from uzbekistan.dynamic_importer import import_conditional_classes

urlpatterns = []
for view in import_conditional_classes('uzbekistan.views', 'views'):
    url_path = f'{view.url_path}/<int:{view.url_relation}>/' if view.url_relation else f'{view.url_path}/'
    urlpatterns.append(path(url_path, view.as_view(), name=f'{view.url_name}'))



# Generated by Django 4.2.7 on 2024-01-31 08:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('uzbekistan', '0002_alter_region_name_oz_alter_region_name_ru_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='Village',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name_uz', models.CharField(max_length=255)),
                ('name_oz', models.CharField(max_length=255)),
                ('name_ru', models.CharField(max_length=255)),
                ('district', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='uzbekistan.district')),
            ],
            options={
                'db_table': 'villages',
                'unique_together': {('name_uz', 'name_oz', 'name_ru', 'district')},
            },
        ),
    ]



# Generated by Django 4.2.7 on 2024-01-25 03:18

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('uzbekistan', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='region',
            name='name_oz',
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name='region',
            name='name_ru',
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name='region',
            name='name_uz',
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name='region',
            name='name_en',
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AlterUniqueTogether(
            name='district',
            unique_together={('name_uz', 'name_oz', 'name_ru', 'region')},
        ),
    ]



from importlib import import_module

from django.conf import settings


def import_conditional_classes(module_name, class_type):
    module = import_module(module_name)
    for class_name, is_enabled in settings.UZBEKISTAN.get(class_type, {}).items():
        if is_enabled and settings.UZBEKISTAN['models'].get(class_name.lower(), False):
            yield getattr(module, f'{class_name.title()}ListAPIView')



# Generated by Django 4.2.7 on 2023-11-03 12:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Region',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name_uz', models.CharField(max_length=255)),
                ('name_oz', models.CharField(max_length=255)),
                ('name_ru', models.CharField(max_length=255)),
                ('name_en', models.CharField(max_length=255)),
            ],
            options={
                'db_table': 'regions',
            },
        ),
        migrations.CreateModel(
            name='District',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name_uz', models.CharField(max_length=255)),
                ('name_oz', models.CharField(max_length=255)),
                ('name_ru', models.CharField(max_length=255)),
                ('region', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='uzbekistan.region')),
            ],
            options={
                'db_table': 'districts',
            },
        ),
    ]



from django.apps import AppConfig
from django.conf import settings


class UzbekistanConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'uzbekistan'

    def ready(self):
        if not settings.UZBEKISTAN.get('models') or not settings.UZBEKISTAN.get('views'):
            raise Exception('UZBEKISTAN settings is not configured properly.')

        for model_name, is_enabled in settings.UZBEKISTAN['models'].items():
            if not is_enabled:
                model_name = model_name.title()
                self.get_model(model_name)._meta.managed = False
                self.get_model(model_name)._meta.abstract = True



from django.conf import settings
from django.db.models import Model, CharField, ForeignKey, CASCADE


class Region(Model):
    name_uz = CharField(max_length=255, unique=True)
    name_oz = CharField(max_length=255, unique=True)
    name_ru = CharField(max_length=255, unique=True)
    name_en = CharField(max_length=255, unique=True)

    class Meta:
        db_table = 'regions'

    def __str__(self):
        return self.name_uz


class District(Model):
    name_uz = CharField(max_length=255)
    name_oz = CharField(max_length=255)
    name_ru = CharField(max_length=255)
    name_en = CharField(max_length=255, null=True, blank=True)
    region = ForeignKey('uzbekistan.Region', on_delete=CASCADE)

    class Meta:
        db_table = 'districts'
        unique_together = ('name_uz', 'name_oz', 'name_ru', 'region')

    def __str__(self):
        return self.name_uz


class Village(Model):
    name_uz = CharField(max_length=255)
    name_oz = CharField(max_length=255)
    name_ru = CharField(max_length=255)
    district = ForeignKey('uzbekistan.District', on_delete=CASCADE)

    class Meta:
        db_table = 'villages'
        unique_together = ('name_uz', 'name_oz', 'name_ru', 'district')

    def __str__(self):
        return self.name_uz


def check_model(model):
    if model._meta.abstract or not settings.UZBEKISTAN['models'].get(model.__name__.lower(), False):
        raise NotImplementedError(
            f"The model '{model}' is not enabled in the current configuration. "
            "Please check that this model is set to True in the 'models' dictionary "
            "of the UZBEKISTAN setting in your settings.py file."

        )
    if model == District and not settings.UZBEKISTAN['models'].get('region', False):
        raise NotImplementedError(
            "The 'District' model requires the 'Region' model to be enabled. "
            "Please ensure that 'Region' is set to True in the 'models' dictionary "
            "of the UZBEKISTAN setting in your settings.py file."
        )

    if model == Village:
        if not settings.UZBEKISTAN['models'].get('region', False):
            raise NotImplementedError(
                "The 'Village' model requires the 'Region' model to be enabled. "
                "Please ensure that 'Region' is set to True in the 'models' dictionary "
                "of the UZBEKISTAN setting in your settings.py file."
            )
        if not settings.UZBEKISTAN['models'].get('district', False):
            raise NotImplementedError(
                "The 'Village' model requires both 'Region' and 'District' models to be enabled. "
                "Please ensure that both are set to True in the 'models' dictionary "
                "of the UZBEKISTAN setting in your settings.py file."
            )



from django.db.models import Q

from uzbekistan.models import Region, District, Village
from django_filters import CharFilter, DateFilter, ChoiceFilter, BooleanFilter, NumberFilter
from django_filters.rest_framework import FilterSet


class RegionFilterSet(FilterSet):
    name = CharFilter(method='filter_by_name')

    @staticmethod
    def filter_by_name(queryset, name, value):
        return queryset.filter(
            Q(name_uz__startswith=value) |
            Q(name_oz__startswith=value) |
            Q(name_ru__startswith=value)
        )

    class Meta:
        model = Region
        fields = ('name',)


class DistrictFilterSet(FilterSet):
    name = CharFilter(method='filter_by_name')

    @staticmethod
    def filter_by_name(queryset, name, value):
        return queryset.filter(
            Q(name_uz__startswith=value) |
            Q(name_oz__startswith=value) |
            Q(name_ru__startswith=value)
        )

    class Meta:
        model = District
        fields = ('name',)


class VillageFilterSet(FilterSet):
    name = CharFilter(method='filter_by_name')

    @staticmethod
    def filter_by_name(queryset, name, value):
        return queryset.filter(
            Q(name_uz__startswith=value) |
            Q(name_oz__startswith=value) |
            Q(name_ru__startswith=value)
        )

    class Meta:
        model = Village
        fields = ('name',)



from setuptools import setup, find_packages

# Define your package name
PACKAGE_NAME = 'uzbekistan'

# Read the long description from your README file
with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name=PACKAGE_NAME,
    version='2.7.2',
    description='Full Database of Uzbekistan Regions, Districts & Quarters with Latin, Cyrillic and Russian versions.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Jakhongir Ganiev',
    author_email='ganiyevuzb@gmail.com',
    url='https://github.com/ganiyevuz/uzbekistan',
    packages=['uzbekistan'],
    include_package_data=True,
    install_requires=[
        'Django',
        'djangorestframework',
        'django-filter',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',  # Choose an appropriate open-source license
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='uzbekistan regions districts',
    python_requires='>=3.9',
)



# Generated by Django 4.2.7 on 2024-01-31 09:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('uzbekistan', '0003_village'),
    ]

    operations = [
        migrations.AddField(
            model_name='district',
            name='name_en',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]



from rest_framework.serializers import ModelSerializer

from uzbekistan.models import Region, District, Village


class RegionModelSerializer(ModelSerializer):
    class Meta:
        model = Region
        fields = '__all__'


class DistrictModelSerializer(ModelSerializer):
    class Meta:
        model = District
        fields = '__all__'


class VillageModelSerializer(ModelSerializer):
    class Meta:
        model = Village
        fields = '__all__'
